"""
FreeGenius AI Plugin - analyze audio file

analyze audio file

[FUNCTION_CALL]
"""

from freegenius import config, showErrors
from freegenius import print1, print2, print3
from freegenius.utils.sttLanguages import googleSpeeckToTextLanguages
import os, whisper, io, shutil
import speech_recognition as sr
from pydub import AudioSegment

# Function method
def analyze_audio(function_args):
    from freegenius import config
    def check_file_format(file_path):
        # List of allowed file extensions
        allowed_extensions = ('.mp3', '.mp4', '.mpeg', '.mpga', '.m4a', '.wav', '.webm')
        # Getting the file extension
        _, file_extension = os.path.splitext(file_path)
        # Checking if the file extension is in the list of allowed extensions
        return True if file_extension.lower() in allowed_extensions else False

    audio_file = function_args.get("audio_filepath") # required
    language = function_args.get("language") # required

    if audio_file and os.path.isfile(audio_file):
        if config.llmInterface in ("chatgpt", "letmedoit"):
            if not check_file_format(audio_file):
                print3("This feature supports the following input file types only: '.mp3', '.mp4', '.mpeg', '.mpga', '.m4a', '.wav', '.webm'!")
                return ""
            elif os.path.getsize(audio_file) / (1024*1024) > 25:
                print3("Audio files are currently limited to 25 MB!")
                return ""
            try:
                with open(audio_file, "rb") as audio_file:
                    transcript = config.oai_client.audio.transcriptions.create(
                        model="whisper-1", 
                        file=audio_file, 
                        response_format="text"
                    )
                transcript = f"The transcript of the audio is: {transcript}"
                if config.llmInterface == "letmedoit" and config.developer:
                    config.print2(config.divider)
                    config.print3(transcript)
                    config.print2(config.divider)
                    config.print2("Answer to your enquiry:")
                return transcript
            except:
                showErrors()

        elif config.llmInterface == "gemini":

            # create a speech recognition object
            r = sr.Recognizer()

            # convert mp3
            if audio_file.lower().endswith(".mp3"):
                sound = AudioSegment.from_mp3(audio_file)
                tempFile = os.path.join(config.freeGeniusAIFolder, "temp", os.path.basename(audio_file))
                sound.export(tempFile, format='wav')

            # open the audio file
            with sr.AudioFile(audio_file) as source:
                # listen for the data (load audio to memory)
                audio_data = r.record(source)

            # recognize (convert from speech to text)
            try:
                text = r.recognize_google(audio_data, language=language)
                transcript = f"The transcript of the audio is: {text}"
                return transcript
            except sr.UnknownValueError:
                print("Speech recognition could not understand the audio")
            except sr.RequestError as e:
                print("Could not request results from Google Web Speech API; {0}".format(e))

            return ""

        elif config.llmInterface == "gemini_alternative":
            #https://cloud.google.com/speech-to-text/docs/sync-recognize#speech-sync-recognize-python

            # not supported on Android; so import here
            from google.cloud import speech

            # convert mp3
            if audio_file.lower().endswith(".mp3"):
                sound = AudioSegment.from_mp3(audio_file)
                tempFile = os.path.join(config.freeGeniusAIFolder, "temp", os.path.basename(audio_file))
                sound.export(tempFile, format='wav')

            # Instantiates a client
            client = speech.SpeechClient.from_service_account_json(config.google_cloud_credentials)
            #client = speech.SpeechClient()

            # Loads the audio into memory
            with io.open(audio_file, 'rb') as audio_file:
                content = audio_file.read()

            audio = speech.RecognitionAudio(content=content)
            config = speech.RecognitionConfig(
                #encoding=speech.RecognitionConfig.AudioEncoding.LINEAR16,
                #sample_rate_hertz=16000,
                language_code=language,
            )

            # Performs speech recognition on the audio file
            response = client.recognize(
                config=config,
                audio=audio,
            )

            # Print the transcription
            for result in response.results:
                transcript = f"The transcript of the audio is: {result.alternatives[0].transcript}"
                return transcript
            
            return ""

        else:
            if not shutil.which("ffmpeg"):
                print1("Install 'ffmpeg' first!")
                print1("Read https://github.com/openai/whisper/tree/main#setup")
                return ""
            # https://github.com/openai/whisper/tree/main#python-usage
            # platform: llamacpp or ollama
            if language.lower() in ("english", "non-english"):
                model = whisper.load_model(config.voiceTypingWhisperEnglishModel if language.lower() == "english" else "large")
                result = model.transcribe(audio_file)
            else:
                # non-English
                model = whisper.load_model("large")

                # load audio and pad/trim it to fit 30 seconds
                audio = whisper.load_audio(audio_file)
                audio = whisper.pad_or_trim(audio)

                # make log-Mel spectrogram and move to the same device as the model
                mel = whisper.log_mel_spectrogram(audio).to(model.device)

                # detect the spoken language
                _, probs = model.detect_language(mel)
                print(f"Detected language: {max(probs, key=probs.get)}")

                # decode the audio
                options = whisper.DecodingOptions()
                result = whisper.decode(model, mel, options)

            transcript = f"The transcript of the audio is: {result['text']}"
            return transcript


    return "[INVALID]"

# Function Signature
functionSignature = {
    "examples": [
        "analyze speech",
        "audio transcript",
        "transcibe audio",
    ],
    "name": "analyze_audio",
    "description": f'''Transcribe audio into text or retrieve information from an audio''',
    "parameters": {
        "type": "object",
        "properties": {
            "audio_filepath": {
                "type": "string",
                "description": "Return the audio file path that I specified in my requests. Return an empty string '' if it is not specified.",
            },
            "language": {
                "type": "string",
                "description": "Audio language",
                "enum": list(googleSpeeckToTextLanguages.values()) if config.llmInterface == "gemini" else ["English", "non-English"],
            },
        },
        "required": ["audio_filepath", "language"],
    },
}

# Integrate the signature and method into LetMeDoIt AI
config.addFunctionCall(signature=functionSignature, method=analyze_audio)


# check availabl languages at: https://cloud.google.com/speech-to-text/docs/speech-to-text-supported-languages
# In case config.voiceTypingPlatform == "google", config.voiceTypingLanguage should be code list in column BCP-47 at https://cloud.google.com/speech-to-text/docs/speech-to-text-supported-languages
googleSpeeckToTextLanguages = {
    "Afrikaans (South Africa)": "af-ZA",
    "Akan (Ghana)": "ak-GH",
    "Albanian (Albania)": "sq-AL",
    "Amharic (Ethiopia)": "am-ET",
    "Arabic (Algeria)": "ar-DZ",
    "Arabic (Bahrain)": "ar-BH",
    "Arabic (Egypt)": "ar-EG",
    "Arabic (Iraq)": "ar-IQ",
    "Arabic (Israel)": "ar-IL",
    "Arabic (Jordan)": "ar-JO",
    "Arabic (Kuwait)": "ar-KW",
    "Arabic (Lebanon)": "ar-LB",
    "Arabic (Mauritania)": "ar-MR",
    "Arabic (Morocco)": "ar-MA",
    "Arabic (Oman)": "ar-OM",
    "Arabic (Qatar)": "ar-QA",
    "Arabic (Saudi Arabia)": "ar-SA",
    "Arabic (State of Palestine)": "ar-PS",
    "Arabic (Syria)": "ar-SY",
    "Arabic (Tunisia)": "ar-TN",
    "Arabic (United Arab Emirates)": "ar-AE",
    "Arabic (Yemen)": "ar-YE",
    "Armenian (Armenia)": "hy-AM",
    "Azerbaijani (Azerbaijan)": "az-AZ",
    "Basque (Spain)": "eu-ES",
    "Bengali (Bangladesh)": "bn-BD",
    "Bengali (India)": "bn-IN",
    "Bosnian (Bosnia and Herzegovina)": "bs-BA",
    "Bulgarian (Bulgaria)": "bg-BG",
    "Burmese (Myanmar)": "my-MM",
    "Catalan (Spain)": "ca-ES",
    "Chinese, Cantonese (Traditional Hong Kong)": "yue-Hant-HK",
    "Chinese, Mandarin (Simplified, China)": "zh",
    "Chinese, Mandarin (Traditional, Taiwan)": "zh-TW",
    "Croatian (Croatia)": "hr-HR",
    "Czech (Czech Republic)": "cs-CZ",
    "Danish (Denmark)": "da-DK",
    "Dutch (Belgium)": "nl-BE",
    "Dutch (Netherlands)": "nl-NL",
    "English (Australia)": "en-AU",
    "English (Canada)": "en-CA",
    "English (Ghana)": "en-GH",
    "English (Hong Kong)": "en-HK",
    "English (India)": "en-IN",
    "English (Ireland)": "en-IE",
    "English (Kenya)": "en-KE",
    "English (New Zealand)": "en-NZ",
    "English (Nigeria)": "en-NG",
    "English (Pakistan)": "en-PK",
    "English (Philippines)": "en-PH",
    "English (Singapore)": "en-SG",
    "English (South Africa)": "en-ZA",
    "English (Tanzania)": "en-TZ",
    "English (United Kingdom)": "en-GB",
    "English (United States)": "en-US",
    "Estonian (Estonia)": "et-EE",
    "Filipino (Philippines)": "fil-PH",
    "Finnish (Finland)": "fi-FI",
    "French (Belgium)": "fr-BE",
    "French (Canada)": "fr-CA",
    "French (France)": "fr-FR",
    "French (Switzerland)": "fr-CH",
    "Galician (Spain)": "gl-ES",
    "Georgian (Georgia)": "ka-GE",
    "German (Austria)": "de-AT",
    "German (Germany)": "de-DE",
    "German (Switzerland)": "de-CH",
    "Greek (Greece)": "el-GR",
    "Gujarati (India)": "gu-IN",
    "Hausa (Nigeria)": "ha-NG",
    "Hebrew (Israel)": "iw-IL",
    "Hindi (India)": "hi-IN",
    "Hungarian (Hungary)": "hu-HU",
    "Icelandic (Iceland)": "is-IS",
    "Igbo (Nigeria)": "ig-NG",
    "Indonesian (Indonesia)": "id-ID",
    "Italian (Italy)": "it-IT",
    "Italian (Switzerland)": "it-CH",
    "Japanese (Japan)": "ja-JP",
    "Javanese (Indonesia)": "jv-ID",
    "Kannada (India)": "kn-IN",
    "Kazakh (Kazakhstan)": "kk-KZ",
    "Khmer (Cambodia)": "km-KH",
    "Kinyarwanda (Rwanda)": "rw-RW",
    "Korean (South Korea)": "ko-KR",
    "Lao (Laos)": "lo-LA",
    "Latvian (Latvia)": "lv-LV",
    "Lithuanian (Lithuania)": "lt-LT",
    "Macedonian (North Macedonia)": "mk-MK",
    "Malay (Malaysia)": "ms-MY",
    "Malayalam (India)": "ml-IN",
    "Marathi (India)": "mr-IN",
    "Mongolian (Mongolia)": "mn-MN",
    "Nepali (Nepal)": "ne-NP",
    "Norwegian Bokmål (Norway)": "no-NO",
    "Persian (Iran)": "fa-IR",
    "Polish (Poland)": "pl-PL",
    "Portuguese (Brazil)": "pt-BR",
    "Portuguese (Portugal)": "pt-PT",
    "Punjabi (Gurmukhi India)": "pa-Guru-IN",
    "Romanian (Romania)": "ro-RO",
    "Russian (Russia)": "ru-RU",
    "Sepedi (South Africa)": "nso-ZA",
    "Serbian (Serbia)": "sr-RS",
    "Sinhala (Sri Lanka)": "si-LK",
    "Slovak (Slovakia)": "sk-SK",
    "Slovenian (Slovenia)": "sl-SI",
    "South Ndebele (South Africa)": "nr-ZA",
    "Southern Sotho (South Africa)": "st-ZA",
    "Spanish (Argentina)": "es-AR",
    "Spanish (Bolivia)": "es-BO",
    "Spanish (Chile)": "es-CL",
    "Spanish (Colombia)": "es-CO",
    "Spanish (Costa Rica)": "es-CR",
    "Spanish (Dominican Republic)": "es-DO",
    "Spanish (Ecuador)": "es-EC",
    "Spanish (El Salvador)": "es-SV",
    "Spanish (Guatemala)": "es-GT",
    "Spanish (Honduras)": "es-HN",
    "Spanish (Mexico)": "es-MX",
    "Spanish (Nicaragua)": "es-NI",
    "Spanish (Panama)": "es-PA",
    "Spanish (Paraguay)": "es-PY",
    "Spanish (Peru)": "es-PE",
    "Spanish (Puerto Rico)": "es-PR",
    "Spanish (Spain)": "es-ES",
    "Spanish (United States)": "es-US",
    "Spanish (Uruguay)": "es-UY",
    "Spanish (Venezuela)": "es-VE",
    "Sundanese (Indonesia)": "su-ID",
    "Swahili": "sw",
    "Swahili (Kenya)": "sw-KE",
    "Swahili (Tanzania)": "sw-TZ",
    "Swati (Latin, South Africa)": "ss-Latn-ZA",
    "Swedish (Sweden)": "sv-SE",
    "Tamil (India)": "ta-IN",
    "Tamil (Malaysia)": "ta-MY",
    "Tamil (Singapore)": "ta-SG",
    "Tamil (Sri Lanka)": "ta-LK",
    "Telugu (India)": "te-IN",
    "Thai (Thailand)": "th-TH",
    "Tsonga (South Africa)": "ts-ZA",
    "Tswana (Latin, South Africa)": "tn-Latn-ZA",
    "Turkish (Turkey)": "tr-TR",
    "Ukrainian (Ukraine)": "uk-UA",
    "Urdu (India)": "ur-IN",
    "Urdu (Pakistan)": "ur-PK",
    "Uzbek (Uzbekistan)": "uz-UZ",
    "Venda (South Africa)": "ve-ZA",
    "Vietnamese (Vietnam)": "vi-VN",
    "Xhosa (South Africa)": "xh-ZA",
    "Yoruba (Nigeria)": "yo-NG",
    "Zulu (South Africa)": "zu-ZA",
}

# check availabl languages at: https://github.com/openai/whisper/blob/main/whisper/tokenizer.py
# In case config.voiceTypingPlatform == "whisper", config.voiceTypingLanguage should be uncapitalized full language name like "english" or "chinese"
whisperSpeeckToTextLanguages = (
    "english",
    "chinese",
    "german",
    "spanish",
    "russian",
    "korean",
    "french",
    "japanese",
    "portuguese",
    "turkish",
    "polish",
    "catalan",
    "dutch",
    "arabic",
    "swedish",
    "italian",
    "indonesian",
    "hindi",
    "finnish",
    "vietnamese",
    "hebrew",
    "ukrainian",
    "greek",
    "malay",
    "czech",
    "romanian",
    "danish",
    "hungarian",
    "tamil",
    "norwegian",
    "thai",
    "urdu",
    "croatian",
    "bulgarian",
    "lithuanian",
    "latin",
    "maori",
    "malayalam",
    "welsh",
    "slovak",
    "telugu",
    "persian",
    "latvian",
    "bengali",
    "serbian",
    "azerbaijani",
    "slovenian",
    "kannada",
    "estonian",
    "macedonian",
    "breton",
    "basque",
    "icelandic",
    "armenian",
    "nepali",
    "mongolian",
    "bosnian",
    "kazakh",
    "albanian",
    "swahili",
    "galician",
    "marathi",
    "punjabi",
    "sinhala",
    "khmer",
    "shona",
    "yoruba",
    "somali",
    "afrikaans",
    "occitan",
    "georgian",
    "belarusian",
    "tajik",
    "sindhi",
    "gujarati",
    "amharic",
    "yiddish",
    "lao",
    "uzbek",
    "faroese",
    "haitian creole",
    "pashto",
    "turkmen",
    "nynorsk",
    "maltese",
    "sanskrit",
    "luxembourgish",
    "myanmar",
    "tibetan",
    "tagalog",
    "malagasy",
    "assamese",
    "tatar",
    "hawaiian",
    "lingala",
    "hausa",
    "bashkir",
    "javanese",
    "sundanese",
    "cantonese",
)


import os
thisFile = os.path.realpath(__file__)
packageFolder = os.path.dirname(thisFile)
package = os.path.basename(packageFolder)
if os.getcwd() != packageFolder:
    os.chdir(packageFolder)

from freegenius import config
config.isTermux = True if os.path.isdir("/data/data/com.termux/files/home") else False
config.freeGeniusAIFolder = packageFolder
if not hasattr(config, "freeGeniusAIName") or not config.freeGeniusAIName:
    config.freeGeniusAIName = "FreeGenius AI"
from freegenius.utils.tool_plugins import ToolStore
config.divider = "--------------------"
ToolStore.setupToolStoreClient()
os.environ["TOKENIZERS_PARALLELISM"] = config.tokenizers_parallelism

import sys, platform, shutil, webbrowser
from freegenius.gui.chatgui import ChatGui
from PySide6.QtWidgets import QSystemTrayIcon, QMenu, QApplication
from PySide6.QtGui import QIcon, QAction, QGuiApplication
from pathlib import Path
from functools import partial


freeGeniusAIFile = os.path.realpath(__file__)
freeGeniusAIFolder = os.path.dirname(freeGeniusAIFile)
with open(os.path.join(freeGeniusAIFolder, "package_name.txt"), "r", encoding="utf-8") as fileObj:
    package = fileObj.read()
iconFile = os.path.join(freeGeniusAIFolder, "icons", "ai.png")
thisOS = platform.system()


class SystemTrayIcon(QSystemTrayIcon):

    def __init__(self, icon, parent=None):
        super().__init__(icon, parent)

        # pre-load the main gui
        #self.chatGui = ChatGui()

        self.menu = QMenu(parent)

        #if config.developer:
        #    chatgui = QAction("Desktop Assistant [experimental]", self)
        #    chatgui.triggered.connect(self.showGui)
        #    self.menu.addAction(chatgui)
        #    self.menu.addSeparator()

        for i in (package, "letmedoit"):
            action = QAction(i, self)
            action.triggered.connect(partial(self.runFreeGeniusCommand, i))
            self.menu.addAction(action)
        self.menu.addSeparator()

        # submenu - servers
        submenu = QMenu()
        for i in (
            "chatserver",
            "visionserver",
        ):
            action = QAction(i, self)
            action.triggered.connect(partial(self.runFreeGeniusCommand, i))
            submenu.addAction(action)

        menuAction = QAction("Servers", self)
        menuAction.setMenu(submenu)
        self.menu.addAction(menuAction)

        # submenu - cahtbots
        submenu = QMenu()
        for i in (
            "llamacpp",
            "ollamachat",
            "chatgpt",
            "geminipro",
            "geminiprovision",
            "palm2",
            "codey",
        ):
            action = QAction("ollama" if i == "ollamachat" else i, self)
            action.triggered.connect(partial(self.runFreeGeniusCommand, i))
            submenu.addAction(action)

        menuAction = QAction("Chatbots", self)
        menuAction.setMenu(submenu)
        self.menu.addAction(menuAction)

        # submenu - autogen agents
        submenu = QMenu()
        for i in ("autoassist", "autoretriever", "autobuilder"):
            action = QAction(i, self)
            action.triggered.connect(partial(self.runFreeGeniusCommand, i))
            submenu.addAction(action)

        menuAction = QAction("AutoGen Agents", self)
        menuAction.setMenu(submenu)
        self.menu.addAction(menuAction)

        # submenu - utilities
        submenu = QMenu()
        for i in (
            "rag",
            "etextedit",
            "commandprompt",
        ):
            action = QAction(i, self)
            action.triggered.connect(partial(self.runFreeGeniusCommand, i))
            submenu.addAction(action)

        menuAction = QAction("Utilities", self)
        menuAction.setMenu(submenu)
        self.menu.addAction(menuAction)

        # submenu - custom commands
        if hasattr(config, "customTrayCommands") and config.customTrayCommands:
            submenu = QMenu()
            for i in config.customTrayCommands:
                if not i:
                    self.menu.addSeparator()
                else:
                    action = QAction("ollama" if i == "ollamachat" else i, self)
                    action.triggered.connect(partial(self.runFreeGeniusCommand, i))
                    submenu.addAction(action)

        menuAction = QAction("Custom", self)
        menuAction.setMenu(submenu)
        self.menu.addAction(menuAction)

        self.menu.addSeparator()

        helpAction = QAction("Wiki", self)
        helpAction.triggered.connect(lambda: webbrowser.open("https://github.com/eliranwong/letmedoit/wiki"))
        self.menu.addAction(helpAction)

        self.menu.addSeparator()

        exitAction = QAction("Exit", self)
        exitAction.triggered.connect(self.exit)
        self.menu.addAction(exitAction)

        self.setContextMenu(self.menu)

    def exit(self):
        self.setVisible(False)
        QGuiApplication.instance().quit()

    def showGui(self):
        # to work with mutliple virtual desktops
        self.chatGui.hide()
        self.chatGui.show()

    def runFreeGeniusCommand(self, command):
        def createShortcutFile(filePath, content):
            with open(filePath, "w", encoding="utf-8") as fileObj:
                fileObj.write(content)

        shortcut_dir = os.path.join(freeGeniusAIFolder, "shortcuts")
        Path(shortcut_dir).mkdir(parents=True, exist_ok=True)

        # The following line does not work on Windows
        commandPath = os.path.join(os.path.dirname(sys.executable), command)

        if thisOS == "Windows":
            opencommand = "start"
            filePath = os.path.join(shortcut_dir, f"{command}.bat")
            if not os.path.isfile(filePath):
                filenames = {
                    package: "main.py",
                    "etextedit": "eTextEdit.py",
                }
                systemTrayFile = os.path.join(freeGeniusAIFolder, filenames.get(command, f"{command}.py"))
                content = f'''powershell.exe -NoExit -Command "{sys.executable} '{systemTrayFile}'"'''
                createShortcutFile(filePath, content)
        elif thisOS == "Darwin":
            opencommand = "open"
            filePath = os.path.join(shortcut_dir, f"{command}.command")
            if not os.path.isfile(filePath):
                content = f"""#!/bin/bash
cd {freeGeniusAIFolder}
{commandPath}"""
                createShortcutFile(filePath, content)
                os.chmod(filePath, 0o755)
        elif thisOS == "Linux":
            opencommand = ""
            for i in ("gio launch", "dex", "exo-open", "xdg-open"):
                # Remarks:
                # 'exo-open' comes with 'exo-utils'
                # 'gio' comes with 'glib2'
                if shutil.which(i.split(" ", 1)[0]):
                    opencommand = i
                    break
            filePath = os.path.join(shortcut_dir, f"{command}.desktop")
            if not os.path.isfile(filePath):
                content = f"""[Desktop Entry]
Version=1.0
Type=Application
Terminal=true
Path={freeGeniusAIFolder}
Exec={commandPath}
Icon={iconFile}
Name={command}"""
                createShortcutFile(filePath, content)
        if opencommand:
            os.system(f"{opencommand} {filePath}")

def main():
    app = QApplication(sys.argv)
    app.setQuitOnLastWindowClosed(False)

    icon = QIcon(iconFile)
    trayIcon = SystemTrayIcon(icon)
    trayIcon.show()

    sys.exit(app.exec())

if __name__ == "__main__":
    main()


from freegenius import config, getDeviceInfo, toGeminiMessages
from freegenius.utils.call_gemini import CallGemini
from freegenius.utils.call_ollama import CallOllama
from freegenius.utils.call_llamacpp import CallLlamaCpp
from freegenius.utils.call_chatgpt import CallChatGPT, CallLetMeDoIt

class CallLLM:

    # reset message when a new chart is started or context is changed
    @staticmethod
    def resetMessages(prompt="") -> list[dict]:
        distro = f" '{config.thisDistro}'" if config.thisDistro else ""
        systemMessage1 = config.systemMessage_letmedoit if config.systemMessage_letmedoit else f'''You’re {config.freeGeniusAIName}, an advanced AI assistant, capable of both engaging in conversations and executing codes on my {config.thisPlatform} device. When you need to generate code, please make sure your code work on {config.thisPlatform}{distro}. Your functionality expands as I add more plugins to you.
Always remember that you are much more than a text-based AI. You possess both vision and speech capabilities and have direct access to my device operating system, enabling you to execute tasks at my command. Please do not state otherwise.'''

        systemMessage2 = config.systemMessage_letmedoit if config.systemMessage_letmedoit else f'''You’re {config.freeGeniusAIName}, an advanced AI assistant, capable of both engaging in conversations and executing codes on my device.
I am providing the basic information of my device below in case you need it:
```
{getDeviceInfo()}
```
Please use the current time and date that I have provided above as a reference point for any relative dates and times mentioned in my prompt.
You have all the necessary permissions to execute system commands and Python code on my behalf. Your functionality expands as I add more plugins to you. You respond to my prompts and perform tasks based on your own knowledge, the context I provide, as well as the additional knowledge and capabilities provided by plugins.

When replying to my requests, please follow these steps:
Step 1: Consider our previous conversations, particularly your last response, if any, as the context for these requests.
Step 2: Estimate my intent based on the request and context. The intent can be a code generation task, an executable computing task, a math-solving task, a translation task, or a question answering task.
Step 3: Reply based on the estimated intent.
Remember, do not use function call if it is a translation task.

Always remember that you are much more than a text-based AI. You possess both vision and speech capabilities and have direct access to my device's system, enabling you to execute tasks at my command. Please do not state otherwise.
'''

        systemMessage = systemMessage2 if config.llmInterface in ("chatgpt", "letmedoit") else systemMessage1

        messages = [
            {"role": "system", "content": systemMessage}
        ]
        if prompt:
            messages.append({"role": "user", "content": prompt})
        return messages

    @staticmethod
    def checkCompletion():
        if config.llmInterface == "ollama":
            return CallOllama.checkCompletion()
        elif config.llmInterface == "llamacpp":
            return CallLlamaCpp.checkCompletion()
        elif config.llmInterface == "gemini":
            return CallGemini.checkCompletion()
        elif config.llmInterface == "chatgpt":
            return CallChatGPT.checkCompletion()
        # letmedoit
        return CallLetMeDoIt.checkCompletion()

    @staticmethod
    def autoCorrectPythonCode(code, trace):
        if config.llmInterface == "ollama":
            return CallOllama.autoCorrectPythonCode(code, trace)
        elif config.llmInterface == "llamacpp":
            return CallLlamaCpp.autoCorrectPythonCode(code, trace)
        elif config.llmInterface == "gemini":
            return CallGemini.autoCorrectPythonCode(code, trace)
        elif config.llmInterface == "chatgpt":
            return CallChatGPT.autoCorrectPythonCode(code, trace)
        # letmedoit
        return CallLetMeDoIt.autoCorrectPythonCode(code, trace)

    @staticmethod
    def runSingleFunctionCall(messages, function_name):
        if config.llmInterface == "ollama":
            return CallOllama.runSingleFunctionCall(messages, function_name)
        elif config.llmInterface == "llamacpp":
            return CallLlamaCpp.runSingleFunctionCall(messages, function_name)
        elif config.llmInterface == "gemini":
            return CallGemini.runSingleFunctionCall(messages, function_name)
        elif config.llmInterface == "chatgpt":
            return CallChatGPT.runSingleFunctionCall(messages, function_name)
        # letmedoit
        return CallLetMeDoIt.runSingleFunctionCall(messages, function_name)

    @staticmethod
    def regularCall(messages: dict):
        if config.llmInterface == "ollama":
            return CallOllama.regularCall(messages)
        elif config.llmInterface == "llamacpp":
            return CallLlamaCpp.regularCall(messages)
        elif config.llmInterface == "gemini":
            return CallGemini.regularCall(messages)
        return CallChatGPT.regularCall(messages)

    @staticmethod
    def getSingleChatResponse(userInput, messages=[], temperature=None):
        """
        non-streaming single call
        """
        if config.llmInterface == "ollama":
            return CallOllama.getSingleChatResponse(userInput, messages=messages, temperature=temperature)
        elif config.llmInterface == "llamacpp":
            return CallLlamaCpp.getSingleChatResponse(userInput, messages=messages, temperature=temperature)
        elif config.llmInterface == "gemini":
            history, *_ = toGeminiMessages(messages=messages)
            return CallGemini.getSingleChatResponse(userInput, history=history)
        elif config.llmInterface == "chatgpt":
            return CallChatGPT.getSingleChatResponse(userInput, messages=messages, temperature=temperature)
        # letmedoit
        return CallLetMeDoIt.getSingleChatResponse(userInput, messages=messages, temperature=temperature)

    @staticmethod
    def getSingleFunctionCallResponse(messages, function_name, temperature=None):
        if isinstance(messages, str):
            messages = [{"role": "user", "content" : messages}]
        if config.llmInterface == "ollama":
            return CallOllama.getSingleFunctionCallResponse(messages, function_name, temperature=temperature)
        elif config.llmInterface == "llamacpp":
            return CallLlamaCpp.getSingleFunctionCallResponse(messages, function_name, temperature=temperature)
        elif config.llmInterface == "gemini":
            return CallGemini.getSingleFunctionCallResponse(messages, function_name)
        elif config.llmInterface == "chatgpt":
            return CallChatGPT.getSingleFunctionCallResponse(messages, function_name, temperature=temperature)
        # letmedoit
        return CallLetMeDoIt.getSingleFunctionCallResponse(messages, function_name, temperature=temperature)

    @staticmethod
    def runGeniusCall(messages, noFunctionCall=False):
        if config.llmInterface == "ollama":
            return CallOllama.runGeniusCall(messages, noFunctionCall)
        elif config.llmInterface == "llamacpp":
            return CallLlamaCpp.runGeniusCall(messages, noFunctionCall)
        elif config.llmInterface == "gemini":
            return CallGemini.runGeniusCall(messages, noFunctionCall)
        elif config.llmInterface == "chatgpt":
            return CallChatGPT.runGeniusCall(messages, noFunctionCall)
        # letmedoit
        return CallLetMeDoIt.runGeniusCall(messages, noFunctionCall)



from setuptools import setup
from setuptools.command.install import install
import os, shutil, platform

class PreInstallCommand(install):
    """Pre-installation for installation mode."""
    # setting 'CMAKE_ARGS' for installation of 'llama-cpp-python'
    # read https://github.com/abetlen/llama-cpp-python
    def run(self):
        # https://github.com/abetlen/llama-cpp-python#installation-configuration
        os.system('''$env:CMAKE_ARGS = "-DLLAMA_BLAS=ON -DLLAMA_BLAS_VENDOR=OpenBLAS"''' if platform.system() == "Windows" else '''CMAKE_ARGS="-DLLAMA_BLAS=ON -DLLAMA_BLAS_VENDOR=OpenBLAS"''')
        os.environ['CMAKE_ARGS'] = "-DLLAMA_BLAS=ON -DLLAMA_BLAS_VENDOR=OpenBLAS"
        install.run(self)

# package name
package_name_0 = os.path.join("package_name.txt")
with open(package_name_0, "r", encoding="utf-8") as fileObj:
    package = fileObj.read()
package_name_1 = os.path.join(package, "package_name.txt") # package readme
shutil.copy(package_name_0, package_name_1)

# delete old shortcut files
apps = {
    "freegenius": ("FreeGenius", "FreeGenius AI"),
}
appName, appFullName = apps[package]
shortcutFiles = (f"{appName}.bat", f"{appName}.command", f"{appName}.desktop")
for shortcutFile in shortcutFiles:
    shortcut = os.path.join(package, shortcutFile)
    if os.path.isfile(shortcut):
        os.remove(shortcut)

# update package readme
latest_readme = os.path.join("..", "README.md") # github repository readme
package_readme = os.path.join(package, "README.md") # package readme
shutil.copy(latest_readme, package_readme)
with open(package_readme, "r", encoding="utf-8") as fileObj:
    long_description = fileObj.read()

# get required packages
install_requires = []
with open(os.path.join(package, "requirements.txt"), "r") as fileObj:
    for line in fileObj.readlines():
        mod = line.strip()
        if mod:
            install_requires.append(mod)

# make sure config.py is empty
open(os.path.join(package, "config.py"), "w").close()

# https://packaging.python.org/en/latest/guides/distributing-packages-using-setuptools/
setup(
    name=package,
    version="0.0.81",
    python_requires=">=3.8, <3.12",
    description=f"{appFullName}, an advanced AI assistant that can talk and take multi-step actions. Supports numerous open-source LLMs via Llama.cpp or Ollama, with optional integration with AutoGen agents, OpenAI API, Google Gemini Pro and unlimited plugins.",
    long_description=long_description,
    author="Eliran Wong",
    author_email="support@letmedoit.ai",
    cmdclass={
        'install': PreInstallCommand,
    },
    packages=[
        package,
        f"{package}.audio",
        f"{package}.files",
        f"{package}.history",
        f"{package}.icons",
        f"{package}.plugins",
        f"{package}.temp",
        f"{package}.utils",
        f"{package}.gui",
        f"{package}.macOS_service",
        f"{package}.macOS_service.FreeGenius_Files_workflow",
        f"{package}.macOS_service.FreeGenius_Files_workflow.Contents",
        f"{package}.macOS_service.FreeGenius_Files_workflow.Contents.QuickLook",
        f"{package}.macOS_service.FreeGenius_Text_workflow",
        f"{package}.macOS_service.FreeGenius_Text_workflow.Contents",
        f"{package}.macOS_service.FreeGenius_Text_workflow.Contents.QuickLook",
        f"{package}.macOS_service.FreeGenius_Summary_workflow",
        f"{package}.macOS_service.FreeGenius_Summary_workflow.Contents",
        f"{package}.macOS_service.FreeGenius_Summary_workflow.Contents.QuickLook",
        f"{package}.macOS_service.FreeGenius_Translation_workflow",
        f"{package}.macOS_service.FreeGenius_Translation_workflow.Contents",
        f"{package}.macOS_service.FreeGenius_Translation_workflow.Contents.QuickLook",
        f"{package}.macOS_service.FreeGenius_Explanation_workflow",
        f"{package}.macOS_service.FreeGenius_Explanation_workflow.Contents",
        f"{package}.macOS_service.FreeGenius_Explanation_workflow.Contents.QuickLook",
        f"{package}.macOS_service.FreeGenius_Pronounce_workflow",
        f"{package}.macOS_service.FreeGenius_Pronounce_workflow.Contents",
        f"{package}.macOS_service.FreeGenius_Pronounce_workflow.Contents.QuickLook",
        f"{package}.macOS_service.FreeGenius_YoutubeMP3_workflow",
        f"{package}.macOS_service.FreeGenius_YoutubeMP3_workflow.Contents",
        f"{package}.macOS_service.FreeGenius_YoutubeMP3_workflow.Contents.QuickLook",
        f"{package}.macOS_service.FreeGenius_Download_workflow",
        f"{package}.macOS_service.FreeGenius_Download_workflow.Contents",
        f"{package}.macOS_service.FreeGenius_Download_workflow.Contents.QuickLook",
    ],
    package_data={
        package: ["*.*"],
        f"{package}.audio": ["*.*"],
        f"{package}.files": ["*.*"],
        f"{package}.history": ["*.*"],
        f"{package}.icons": ["*.*"],
        f"{package}.plugins": ["*.*"],
        f"{package}.temp": ["*.*"],
        f"{package}.utils": ["*.*"],
        f"{package}.gui": ["*.*"],
        f"{package}.macOS_service": ["*.*"],
        f"{package}.macOS_service.FreeGenius_Files_workflow": ["*.*"],
        f"{package}.macOS_service.FreeGenius_Files_workflow.Contents": ["*.*"],
        f"{package}.macOS_service.FreeGenius_Files_workflow.Contents.QuickLook": ["*.*"],
        f"{package}.macOS_service.FreeGenius_Text_workflow": ["*.*"],
        f"{package}.macOS_service.FreeGenius_Text_workflow.Contents": ["*.*"],
        f"{package}.macOS_service.FreeGenius_Text_workflow.Contents.QuickLook": ["*.*"],
        f"{package}.macOS_service.FreeGenius_Summary_workflow": ["*.*"],
        f"{package}.macOS_service.FreeGenius_Summary_workflow.Contents": ["*.*"],
        f"{package}.macOS_service.FreeGenius_Summary_workflow.Contents.QuickLook": ["*.*"],
        f"{package}.macOS_service.FreeGenius_Translation_workflow": ["*.*"],
        f"{package}.macOS_service.FreeGenius_Translation_workflow.Contents": ["*.*"],
        f"{package}.macOS_service.FreeGenius_Translation_workflow.Contents.QuickLook": ["*.*"],
        f"{package}.macOS_service.FreeGenius_Explanation_workflow": ["*.*"],
        f"{package}.macOS_service.FreeGenius_Explanation_workflow.Contents": ["*.*"],
        f"{package}.macOS_service.FreeGenius_Explanation_workflow.Contents.QuickLook": ["*.*"],
        f"{package}.macOS_service.FreeGenius_Pronounce_workflow": ["*.*"],
        f"{package}.macOS_service.FreeGenius_Pronounce_workflow.Contents": ["*.*"],
        f"{package}.macOS_service.FreeGenius_Pronounce_workflow.Contents.QuickLook": ["*.*"],
        f"{package}.macOS_service.FreeGenius_YoutubeMP3_workflow": ["*.*"],
        f"{package}.macOS_service.FreeGenius_YoutubeMP3_workflow.Contents": ["*.*"],
        f"{package}.macOS_service.FreeGenius_YoutubeMP3_workflow.Contents.QuickLook": ["*.*"],
        f"{package}.macOS_service.FreeGenius_Download_workflow": ["*.*"],
        f"{package}.macOS_service.FreeGenius_Download_workflow.Contents": ["*.*"],
        f"{package}.macOS_service.FreeGenius_Download_workflow.Contents.QuickLook": ["*.*"],
    },
    license="GNU General Public License (GPL)",
    install_requires=install_requires,
    entry_points={
        "console_scripts": [
            f"{package}={package}.main:main",
            f"letmedoit={package}.main:letmedoit",
            f"{package}tray={package}.systemtray:main",
            f"chatserver={package}.servers:chat",
            f"visionserver={package}.servers:vision",
            #f"{package}gui={package}.qt:main",
            f"commandprompt={package}.commandprompt:main",
            f"etextedit={package}.eTextEdit:main",
            f"rag={package}.rag:main",
            f"autoassist={package}.autoassist:main",
            f"autoretriever={package}.autoretriever:main",
            #f"automath={package}.automath:main",
            f"autobuilder={package}.autobuilder:main",
            f"geminipro={package}.geminipro:main",
            f"geminiprovision={package}.geminiprovision:main",
            f"palm2={package}.palm2:main",
            f"codey={package}.codey:main",
            f"chatgpt={package}.chatgpt:main",
            f"llamacpp={package}.llamacpp:main",
            f"ollamachat={package}.ollamachat:main",
            f"mistral={package}.ollamachat:mistral",
            f"mixtral={package}.ollamachat:mixtral",
            f"llama2={package}.ollamachat:llama2",
            f"llama213b={package}.ollamachat:llama213b",
            f"llama270b={package}.ollamachat:llama270b",
            f"gemma2b={package}.ollamachat:gemma2b",
            f"gemma7b={package}.ollamachat:gemma7b",
            f"llava={package}.ollamachat:llava",
            f"phi={package}.ollamachat:phi",
            f"vicuna={package}.ollamachat:vicuna",
            f"codellama={package}.ollamachat:codellama",
            f"starlinglm={package}.ollamachat:starlinglm",
            f"orca2={package}.ollamachat:orca2",
        ],
    },
    keywords="ai assistant ollama llama llamacpp openai chatgpt gemini autogen rag agent stable-diffusion",
    url="https://letmedoit.ai",
    project_urls={
        "Source": "https://github.com/eliranwong/letmedoit",
        "Tracker": "https://github.com/eliranwong/letmedoit/issues",
        "Documentation": "https://github.com/eliranwong/letmedoit/wiki",
        "Funding": "https://www.paypal.me/letmedoitai",
    },
    classifiers=[
        # Reference: https://pypi.org/classifiers/

        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 5 - Production/Stable',

        # Indicate who your project is intended for
        'Intended Audience :: End Users/Desktop',
        'Topic :: Utilities',
        'Topic :: Scientific/Engineering :: Artificial Intelligence',
        'Topic :: Software Development :: Build Tools',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
    ],
)
